#ifndef _EXPRESSION_H_
#define _EXPRESSION_H_ 1

#include <list>
#include "tokenizer.h"

//
// Forward definitions
//

typedef struct _expr_item               EXPR_ITEM;
typedef std::list<EXPR_ITEM *>          LIST_EXPR_ITEM;
typedef LIST_EXPR_ITEM::iterator        LIST_EXPR_ITEM_IT;
typedef struct _expression              EXPRESSION;

//
// precedence_array and precedence_array_cache
//

typedef enum {
    OP_NONE = -1,
    OP_0 = 0,
    OP_1,
    OP_2,
    OP_MAX
} OPERATOR_PRIORITY;

typedef std::list<LIST_EXPR_ITEM *>     PRECEDENCE_ITEM;
typedef PRECEDENCE_ITEM::iterator       PRECEDENCE_ITEM_IT;

//
// Operand value
//

typedef enum {
    OVT_VALUE_DOUBLE = 1,
    OVT_VALUE_EXPR
} OPERAND_VALUE_TYPE;

typedef struct _operand_value {
    OPERAND_VALUE_TYPE      t;
    union {
        double              d;
        EXPRESSION          *expr;              // pointer to another expression in parentheses
        // there can be:
        // - value string
        // - variable
        // - function call
    };
} OPERAND_VALUE;

//
// CUO
//

typedef enum {
    CIT_TOKEN_OPERATOR = 1,
    CIT_OPERAND_VALUE
} CUO_ITEM_TYPE;

typedef struct _cuo_item {
    CUO_ITEM_TYPE               t;
    union {
        TOKEN_OPERATOR_TYPE     uop;            // unary operator
        OPERAND_VALUE           value;
    };
} CUO_ITEM;

typedef std::list<CUO_ITEM *>           LIST_CUO_ITEM;
typedef LIST_CUO_ITEM::iterator         LIST_CUO_ITEM_IT;

//
// Operand
//

typedef enum {
    OT_VALUE = 1,
    OT_CUO,
    OT_LIST
} OPERAND_TYPE;

typedef struct _operand {
    OPERAND_TYPE                t;
    union {
        OPERAND_VALUE           value;
        LIST_CUO_ITEM           *cuo;
        LIST_EXPR_ITEM          *list;          // pointer to another list in precedence_array
    };
} OPERAND;

typedef std::list<OPERAND *>            LIST_OPERAND;
typedef LIST_OPERAND::iterator          LIST_OPERAND_IT;

//
// List in precedence_array and precedence_array_cache
//

typedef enum {
    EIT_OPERAND = 1,
    EIT_OPERATOR,
} EXPR_ITEM_TYPE;

typedef struct _expr_item {
    EXPR_ITEM_TYPE              t;
    union {
        OPERAND                 operand;
        TOKEN_OPERATOR_TYPE     bop;            // binary operator
    };
} EXPR_ITEM;

//
// Expression context
//

typedef struct _expression {
    PRECEDENCE_ITEM     precedence_array[OP_MAX];
    LIST_EXPR_ITEM      *precedence_array_cache[OP_MAX];
    LIST_OPERAND        list_cuo;                               // for printing list
    LIST_OPERAND        list_parentheses;
    LIST_OPERAND        list_functions;
} EXPRESSION;

#endif /* _EXPRESSION_H_ */
