#include <stdio.h>
#include "printer.h"

const char *token_operator_type_str(TOKEN_OPERATOR_TYPE ot)
{
    switch (ot) {
        case TOT_PLUS:
            return "+";
        case TOT_MINUS:
            return "-";
        case TOT_MUL:
            return "*";
        case TOT_DIV:
            return "/";
        case TOT_LTHAN:
            return "<";
        case TOT_GTHAN:
            return ">";
        default:
            break;
    }
    return "unknown";
}

int print_operand_value(OPERAND_VALUE *operand_value)
{
    switch (operand_value->t) {
        case OVT_VALUE_DOUBLE:
            printf("%lf", operand_value->d);
            break;
        case OVT_VALUE_EXPR:
            printf("EXPR(0x%p)", operand_value->expr);
            break;
    }
    return 0;
}

int print_operand(OPERAND *operand)
{
    switch (operand->t) {
        case OT_VALUE:
            print_operand_value(&operand->value);
            break;
        case OT_CUO:
            printf("CUO(0x%p)", operand->cuo);
            break;
        case OT_LIST:
            printf("LIST(0x%p)", operand->list);
            break;
    }
    return 0;
}

int print_expr_item(EXPR_ITEM *expr_item)
{
    switch (expr_item->t) {
        case EIT_OPERAND:
            print_operand(&(expr_item->operand));
            break;
        case EIT_OPERATOR:
            printf("%s", token_operator_type_str(expr_item->bop));
            break;
    }
    return 0;
}

int print_list_expr_item(LIST_EXPR_ITEM *list_expr_item)
{
    LIST_EXPR_ITEM_IT   it;

    printf("0x%p{", list_expr_item);
    for (it = list_expr_item->begin(); it != list_expr_item->end(); it++) {
        if (it != list_expr_item->begin())
            printf(", ");
        print_expr_item(*it);
    }
    printf("} ");
    return 0;
}

int print_cuo_item(CUO_ITEM *cuo_item)
{
    switch (cuo_item->t) {
        case CIT_TOKEN_OPERATOR:
            printf("%s", token_operator_type_str(cuo_item->uop));
            break;
        case CIT_OPERAND_VALUE:
            print_operand_value(&cuo_item->value);
            break;
    }
    return 0;
}

int print_list_cuo_item(LIST_CUO_ITEM *list_cuo_item)
{
    LIST_CUO_ITEM_IT    it;

    printf("0x%p{", list_cuo_item);
    for (it = list_cuo_item->begin(); it != list_cuo_item->end(); it++) {
        if (it != list_cuo_item->begin())
            printf(", ");
        print_cuo_item(*it);
    }
    printf("}");
    return 0;
}

int print_expression(EXPRESSION *expression)
{
    int idx;

    // print main expression

    printf("\nMain expression:\n");
    {
        PRECEDENCE_ITEM     *precedence_array = expression->precedence_array;
        PRECEDENCE_ITEM     *precedence_item;
        PRECEDENCE_ITEM_IT  it;

        for (idx = 0; idx < OP_MAX; idx++) {
            printf("Precedence %d: ", idx);
            precedence_item = &(precedence_array[idx]);
            for (it = precedence_item->begin(); it != precedence_item->end(); it++)
                print_list_expr_item(*it);
            printf("\n");
        }
    }

    // print CUO lists

    printf("\nCUO lists:\n");
    {
        LIST_OPERAND    *list_cuo = & expression->list_cuo;
        LIST_OPERAND_IT it;

        for (it = list_cuo->begin(); it != list_cuo->end(); it++) {
            print_list_cuo_item((*it)->cuo);
            printf("\n");
        }
    }

    return 0;
}

